CHECKOUT_ORDER_ID = 'checkout_order_id'
CHECKOUT_FINGERPRINT_SESSION_ID = 'cybersource_fingerprint_session_id'

# Default code for the email to send after successful checkout
DECISION_ACCEPT = 'ACCEPT'
